import sys
import time


class CodingDrama:
    def __init__(self, filename):
        self.CTL_CHARS = ['%', '>', '?', ':', '$', '*']

        self.lines = self._readfile(filename)
        self.tokens = self._tokenize()

    def _readfile(self, filename):
        with open(filename, 'r') as f:
            return f.readlines()

    def _tokenize(self):
        string = '\n'.join(self.lines) + '\n'
        tokens = []

        def _text(string):
            tok = ''
            if string[0] in self.CTL_CHARS+['\n']:
                return None, string
            for c in string:
                if c not in self.CTL_CHARS+['\n']:
                    tok += c
                else:
                    return tok, string[len(tok):]

        def _ctl(string):
            tok = ''
            if string[0] not in self.CTL_CHARS:
                return None, string
            for c in string:
                if c not in (' ', '\n'):
                    tok += c
                else:
                    return tok.strip(), string[len(tok):]

        while len(string):
            text, string = _text(string)
            if text is not None:
                tokens.append(text)
                continue

            ctl, string = _ctl(string)
            if ctl is not None:
                tokens.append(ctl)
                continue

            if string[0] == '\n':
                tokens.append(string[0])
                string = string[1:]
                continue


            raise SyntaxError(f"Unexpected character {string[0]}")

        return tokens

    def walk(self):

        def ccode(name):
            return {
                'BLACK'       : 30,
                'DARK_RED'    : 31,
                'DARK_GREEN'  : 32,
                'DARK_YELLOW' : 33,
                'DARK_BLUE'   : 34,
                'DARK_MAGENTA': 35,
                'DARK_CYAN'   : 36,
                'GREY'        : 37,
                'DARK_GREY'   : 90,
                'RED'         : 91,
                'GREEN'       : 92,
                'YELLOW'      : 93,
                'BLUE'        : 94,
                'MAGENTA'     : 95,
                'CYAN'        : 96,
                'WHITE'       : 97,

                'BOLD'        : 1,
                'FAINT'       : 2,
                'UNDERLINE'   : 4,
                'BLINK'       : 5,
                'FRAME'       : 51,
                'CIRCLE'      : 52,

                'ENDC'        : 0
            }.get(name, 0)

        def typewriter(text, speed):
            for c in text:
                print(c, end='')
                sys.stdout.flush()
                time.sleep(1/speed)

        SPEED = 15

        for tok in self.tokens:
            c = tok[0]
            a = tok[1:]

            if c == ':':
                print(f"\033[{ccode(a.upper())}m", end='')
            elif c == '$':
                time.sleep(int(a))
            elif c == '%':
                SPEED = int(a)
            else:
                text = f"{c}{a}"
                typewriter(text, speed=SPEED)


